@@ -21,6 +21,7 @@
 Built as a sets of pep8 checks using flake8.
 """
 
+import ConfigParser
 import gettext
 import imp
 import logging
@@ -31,6 +32,9 @@ import sys
 import tokenize
 import traceback
 
+import d2to1.util
+import pep8
+
 # Don't need this for testing
 logging.disable('LOG')
 
@@ -274,20 +278,6 @@ def hacking_import_alphabetical(logical_line, blank_lines, previous_logical,
                        % (split_previous[1], split_line[1]))
 
 
-@flake8ext
-def hacking_import_no_db_in_virt(logical_line, filename):
-    """Check for db calls from nova/virt
-
-    As of grizzly-2 all the database calls have been removed from
-    nova/virt, and we want to keep it that way.
-
-    H307
-    """
-    if "nova/virt" in filename and not filename.endswith("fake.py"):
-        if logical_line.startswith("from nova import db"):
-            yield (0, "H307: nova.db import not allowed in nova/virt/*")
-
-
 def is_docstring(physical_line, previous_logical):
     """Return True if found docstring
     'A docstring is a string literal that occurs as the first statement in a
@@ -550,22 +540,35 @@ def hacking_not_in(logical_line):
                 yield (logical_line.find('not'), "H902: Use the 'not in' "
                        "operator for collection membership evaluation")
 
+_has_run_registry = dict()
 
-def once_git_check_commit_title():
-    """Check git commit messages.
 
-    nova HACKING recommends not referencing a bug or blueprint in first line,
-    it should provide an accurate description of the change
-    H801
-    H802 Title limited to 72 chars
-    """
-    #Get title of most recent commit
+class GlobalCheck(object):
+    """Base class for checks that should be run only once."""
 
-    subp = subprocess.Popen(['git', 'log', '--no-merges', '--pretty=%s', '-1'],
-                            stdout=subprocess.PIPE)
-    title = subp.communicate()[0]
-    if subp.returncode:
-        raise Exception("git log failed with code %s" % subp.returncode)
+    version = '0.0.1'
+
+    def run(self):
+        """Make run a no-op if run() has been called before.
+
+        Store in a global registry the list of checks we've run. If we have
+        run that one before, just skip doing anything the subsequent times.
+        This way, since pep8 is file/line based, we don't wind up re-running
+        a check on a git commit message over and over again.
+        """
+        global _has_run_registry
+        if self.name not in _has_run_registry.keys():
+            _has_run_registry[self.name] = True
+            ret = self.run_once()
+            if ret is not None:
+                for r in ret:
+                    yield ret
+
+    def run_once(self):
+        pass
+
+
+class GitCheck(GlobalCheck):
 
     #From https://github.com/openstack/openstack-ci-puppet
     #       /blob/master/modules/gerrit/manifests/init.pp#L74
@@ -575,29 +578,100 @@ def once_git_check_commit_title():
                     '([Bb]lue[Pp]rint|[Bb][Pp])[\s\#:]*([A-Za-z0-9\\-]+)')
     GIT_REGEX = re.compile(git_keywords)
 
-    error = False
-    #NOTE(jogo) if match regex but over 3 words, acceptable title
-    if GIT_REGEX.search(title) is not None and len(title.split()) <= 3:
-        print ("H801: git commit title ('%s') should provide an accurate "
-               "description of the change, not just a reference to a bug "
-               "or blueprint" % title.strip())
-        error = True
-    # HACKING.rst recommends commit titles 50 chars or less, but enforces
-    # a 72 character limit
-    if len(title.decode('utf-8')) > 72:
-        print ("H802: git commit title ('%s') should be under 50 chars"
-               % title.strip())
-        error = True
-    return error
+    def __init__(self, tree, *args):
+        pass
+
+    def _get_commit_title(self):
+        if not os.path.exists('.git'):
+            return None
+
+        #Get title of most recent commit
+        subp = subprocess.Popen(
+            ['git', 'log', '--no-merges', '--pretty=%s', '-1'],
+            stdout=subprocess.PIPE)
+        title = subp.communicate()[0]
+        if subp.returncode:
+            raise Exception("git log failed with code %s" % subp.returncode)
+        return title
+
+
+class OnceGitCheckCommitTitleBug(GitCheck):
+    """Check git commit messages for bugs.
+
+    nova HACKING recommends not referencing a bug or blueprint in first line,
+    it should provide an accurate description of the change
+    H801
+    """
+    name = "GitCheckCommitTitleBug"
+
+    def __init__(self, tree, *args):
+        pass
+
+    def run_once(self):
+        title = self._get_commit_title()
+
+        #NOTE(jogo) if match regex but over 3 words, acceptable title
+        if (title and self.GIT_REGEX.search(title) is not None
+                and len(title.split()) <= 3):
+            return(1, 0,
+                   "H801: git commit title ('%s') should provide an accurate "
+                   "description of the change, not just a reference to a bug "
+                   "or blueprint" % title.strip(), self.name)
+
+
+class OnceGitCheckCommitTitleLength(GitCheck):
+    """Check git commit message length.
+
+    HACKING.rst recommends commit titles 50 chars or less, but enforces
+    a 72 character limit
+
+    H802 Title limited to 72 chars
+    """
+    name = "GitCheckCommitTitleLength"
+
+    def __init__(self, tree, *args):
+        pass
+
+    def run_once(self):
+        title = self._get_commit_title()
+
+        if title and len(title.decode('utf-8')) > 72:
+            return(
+                1, 0,
+                "H802: git commit title ('%s') should be under 50 chars"
+                % title.strip(),
+                self.name)
+
+
+class ProxyChecks(GlobalCheck):
+    """Provide a mechanism for locally defined checks."""
+
+    name = 'ProxyChecker'
+
+    def __init__(self, tree, *args):
+        pass
 
     @classmethod
     def add_options(cls, parser):
-        parser.add_option('--git-message', action='store_true',
-                          default=False, dest='git_message',
-                          help="Check for well-formed git commit messages")
-        parser.config_options.append('git-message')
+        # Abusing this method because of when it gets called
+        if not os.path.exists('tox.ini'):
+            return
+        tox_ini = ConfigParser.RawConfigParser()
+        tox_ini.read('tox.ini')
+        if not tox_ini.has_section('hacking'):
+            return
 
-    @classmethod
-    def parse_options(cls, options):
-        if options.git_message:
-            once_git_check_commit_title()
+        # We're looking for local checks, so we need to include the local
+        # dir in the search path
+        sys.path.append('.')
+        if tox_ini.has_option('hacking', 'local-check'):
+            for check_path in set(
+                    tox_ini.get('hacking', 'local-check').split(",")):
+                if check_path.strip():
+                    checker = d2to1.util.resolve_name(check_path)
+                    pep8.register_check(checker)
+        if tox_ini.has_option('hacking', 'local-check-factory'):
+            factory = d2to1.util.resolve_name(
+                tox_ini.get('hacking', 'local-check-factory'))
+            factory(pep8.register_check)
+        sys.path.pop()
